/*
Echo string back to user.
Uses a lookup table of terminating characters.
*/
#include <iostream>
#include <string>

/** A character lookup table, returning true for the
  characters listed in the "stopRead" string. */
class lookupTable {
public:
	enum {nChar=256};
	bool stopHere[nChar];
	lookupTable(const char *stopRead) {
		for (int i=0;i<nChar;i++) stopHere[i]=false;
		for (const char *p=stopRead;*p!=0;p++)
			stopHere[(int)(*p)]=true;
	}
	bool stopAt(char c) const {
		return stopHere[(int)(unsigned char)c];
	}
};

/** Read a string up to some terminating characters 
  listed in the lookupTable. */
std::string readString(std::istream &in,const lookupTable &table)
{
	std::string s;
	while (1) {
		char c=in.get(); /* read a char */
		if (!in) break; /* no more data to read */
		if (table.stopAt(c)) /* hit a stop character? */
			return s; /* done reading--return */
		
		s+=c; /* add char to string */
	}
}

int main() {
	std::string s;
	std::cout<<"Please enter your name, followed by x or newline or quote\n";
	static const lookupTable table("x;'\"\n");
	s=readString(std::cin,table);
	std::cout<<"Hello, "<<s<<"\n";
	std::cout<<"Your name is "<<s.size()<<" bytes long\n";
	return 0;
}
